{*************************************************************************
                Copyright (c) PilotLogic Software House

  Package pl_ExControls
  This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the 
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 * 
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****
 
***************************************************************************}

Unit TplLCDLineUnit;

{$MODE objfpc}{$H+}

Interface

Uses
   SysUtils, Classes, Controls, Graphics,
   Dialogs,
   TplLCDLineUnit_Char;

Type
   TFrameStyle = (Relief, None, Lowered, Raised);
   TFrameColorStyle = (stWindows, stColor);
   TFrameHeight = (double, single);
   TCellShape = (stSquare, stRound);

Type

   TplLCDLine = Class(TGraphicControl)
   Private
      FBitMap: TBitMap;
      FCellSize: integer;
      FCellSpace: integer;
      FColCount: integer;
      FRowCount: integer;
      FCharCount: integer;
      FGlobalColCount: integer;
      FCharWidth: integer;
      FFrameSize: integer;
      FBoardWidth: integer;
      FBoardHeight: integer;
      FLEDWidth: integer;
      FLEDHeight: integer;
      FFrameColor: TColor;
      FBoardColor: TColor;
      FCellColorOn: TColor;
      FCellColorOff: TColor;
      FBWidth: integer;
      FBHeight: integer;
      FOffset: longint;
      FLenText: integer;
      FText: String;
      FCharSpace: boolean;
      FCellsOn: Array Of Array Of boolean;
      FCountOn: integer;
      FAutoSize: boolean;
      FFrameStyle: TFrameStyle;
      FFrameHeight: TFrameHeight;
      FFrameColorStyle: TFrameColorStyle;
      FCellShape: TCellShape;
      Procedure SetCellShape(Const Value: TCellShape);
      Procedure SetFrameColorStyle(Const Value: TFrameColorStyle);
      Procedure SetFrameHeight(Const Value: TFrameHeight);
      Procedure SetFrameStyle(Const Value: TFrameStyle);
      Procedure SetLAutoSize(Const Value: boolean);
      Function GetCellsOn(Row, Col: integer): boolean;
      Procedure SetCellsOn(Row, Col: integer; Const Value: boolean);
      Procedure SetCharSpace(Const Value: boolean);
      Function GetCharCount: longint;
      Function GetGlobalColCount: longint;
      Procedure SetOffset(Const Value: longint);
      Procedure SetText(Const Value: String);
      Procedure SetCellColorOff(Const Value: TColor);
      Procedure SetCelColorlOn(Const Value: TColor);
      Procedure SetBoardColor(Const Value: TColor);
      Procedure SetFrameColor(Const Value: TColor);
      Procedure SetFrameSize(Const Value: integer);
      Procedure SetCellSize(Const Value: integer);
      Procedure SetCellSpace(Const Value: integer);
      Procedure DrawCalc;
      Procedure DrawBorder;
      Procedure DrawGrid;
      Procedure DrawSpace;
      Procedure DrawText;
      Procedure DrawChar(Row, Col, NChar: integer);
      Procedure DrawCell(Row, Col: integer; ColorCell: TColor);
      Procedure DrawShadow(StartP, EndP: TPoint; LineColor1, LineColor2: TColor);
      Procedure OutBorder;
      Procedure DrawCellsOn;
   Protected
      Procedure Paint; Override;
   Public
      Constructor Create(AOwner: TComponent); Override;
      Destructor Destroy; Override;
      Property Offset: longint Read FOffset Write SetOffset;
      Property CharCount: longint Read GetCharCount;
      Property GlobalColCount: longint Read GetGlobalColCount;
      Property ColCount: integer Read FColCount;
      Property RowCount: integer Read FRowCount;
      Property CellsOn[Row, Col: integer]: boolean Read GetCellsOn Write SetCellsOn;
   Published
      Property CellSize: integer Read FCellSize Write SetCellSize;
      Property CellSpace: integer Read FCellSpace Write SetCellSpace;
      Property FrameSize: integer Read FFrameSize Write SetFrameSize;
      Property FrameColor: TColor Read FFrameColor Write SetFrameColor;
      Property BoardColor: TColor Read FBoardColor Write SetBoardColor;
      Property CellColorOn: TColor Read FCellColorOn Write SetCelColorlOn;
      Property CellColorOff: TColor Read FCellColorOff Write SetCellColorOff;
      Property Text: String Read FText Write SetText;
      Property CharSpace: boolean Read FCharSpace Write SetCharSpace Default False;
      Property AutoSize: boolean Read FAutoSize Write SetLAutoSize Default False;
      Property FrameStyle: TFrameStyle Read FFrameStyle Write SetFrameStyle Default Relief;
      Property FrameHeight: TFrameHeight Read FFrameHeight Write SetFrameHeight Default double;
      Property FrameColorStyle: TFrameColorStyle Read FFrameColorStyle Write SetFrameColorStyle Default stWindows;
      Property CellShape: TCellShape Read FCellShape Write SetCellShape Default stSquare;
      Property Visible;
      Property ShowHint;

      Property OnClick;
      Property OnDblClick;
      Property OnPaint;
      Property OnResize;
      Property OnChangeBounds;
      Property OnContextPopup;
      Property OnEndDrag;
      Property OnMouseDown;
      Property OnMouseUp;
      Property OnMouseEnter;
      Property OnMouseLeave;
      Property OnMouseMove;
      Property OnMouseWheel;
      Property OnMouseWheelDown;
      Property OnMouseWheelUp;
      Property OnMouseWheelHorz;
      Property OnMouseWheelLeft;
      Property OnMouseWheelRight;
      Property OnDragDrop;
      Property OnDragOver;
      Property OnStartDrag;
      Property OnStartDock;
      Property OnEndDock;
   End;

Implementation

Constructor TplLCDLine.Create(AOwner: TComponent);
Begin
   Inherited Create(AOwner);
   ControlStyle := ControlStyle + [csOpaque];
   FBitMap := TBitMap.Create;

   SetInitialBounds(0, 0, 156, 76);

   FColCount := 5;
   FRowCount := 7;
   FCellSize := 4;
   FCellSpace := 1;
   FFrameSize := 8;
   FBoardWidth := 4;
   FOffset := 0;
   FFrameColor  := clBtnFace;
   FBoardColor  := clBlack;
   FCellColorOn := clLime;
   FCellColorOff:= clGreen;

   FCountOn := 255;  
   LoadChar;
End;

Destructor TplLCDLine.Destroy;
Begin
   FBitMap.Free;
   FCellsOn := Nil;
   Inherited Destroy;
End;

Procedure TplLCDLine.DrawCalc;
Var
   WidthMin: integer;
   HeightMin: integer;
   Len: integer;
Begin

   FCharWidth := (CellSize * FColCount) + (CellSpace * (FColCount + 1));

   If AutoSize Then
   Begin
      Len := Length(Text);
      If Len = 0 Then
         Len := 1;
      FCharCount := Len;
   End
   Else
      FCharCount := ((Width - (2 * FrameSize)) + CellSize) Div (FCharWidth + CellSize);

   FGlobalColCount := (FCharCount * FColCount) + (FCharCount - 1);

   FLEDWidth := (FGlobalColCount * CellSize) + ((FGlobalColCount - 1) * CellSpace);
   FLEDHeight := (FRowCount * (CellSize + CellSpace)) - 1;

   If AutoSize Then
   Begin
      FBoardWidth := 1;
      FBoardHeight := 1;
      Width := FLEDWidth + (2 * FrameSize) + (2 * FBoardWidth);
      Height := FLEDHeight + (2 * FrameSize) + (2 * FBoardWidth);
   End
   Else
   Begin
      FBoardWidth := (Width - (2 * FrameSize) - FLEDWidth) Div 2;
      FBoardHeight := (Height - (2 * FrameSize) - FLEDHeight) Div 2;
   End;

   WidthMin := (2 * FrameSize) + FCharWidth;
   If Width < WidthMin Then
      Width := WidthMin;

   HeightMin := (2 * FrameSize) + ((FRowCount * (CellSize + CellSpace)) - 1) + 2;
   If Height < HeightMin Then
      Height := HeightMin;

   FBWidth := FLEDWidth + (2 * FBoardWidth);
   FBHeight := (fRowCount * (CellSize + CellSpace)) + (2 * FBoardHeight);

   FBitMap.Width := Width;
   FBitMap.Height := Height;

   SetLength(FCellsOn, FRowCount, FCountOn);
End;

Procedure TplLCDLine.DrawBorder;
Var
   FStart, FEnd: TPoint;
   BStart, BEnd: TPoint;
   Color1, Color2, Color3, Color4: TColor;
   C: longint;
   R, G, B: integer;
Const
   K1 = 4.637;
   K2 = 1.364;
   K3 = 1.372093;
   K4 = 2.088495;
Begin

   BStart.X := FrameSize;
   BStart.Y := FrameSize;
   BEnd.X := Width - FrameSize;
   BEnd.Y := Height - FrameSize;

   With FBitMap.Canvas Do
   Begin
      Brush.Color := FrameColor;
      Pen.Color := FrameColor;
      Rectangle(0, 0, Width, Height);
      Brush.Color := BoardColor;
      Pen.Color := BoardColor;
      Rectangle(BStart.X, BStart.Y, BEnd.X, BEnd.Y);
   End;

   If FrameStyle = None Then
      Exit;

   If FrameColorStyle = stWindows Then
      C := ColorToRGB(clBtnFace)
   Else
      C := ColorToRGB(FrameColor);

   R := RED(C);
   G := GREEN(C);
   B := BLUE(C);

   If FrameColorStyle = stWindows Then
      Color1 := clWhite
   Else
      Color1 := RGBToColor(Round(R / K1 + 200), Round(G / K1 + 200), Round(B / K1 + 200));

   Color2 := RGBToColor(Round(R / K2 + 68), Round(G / K2 + 68), Round(B / K2 + 68));
   Color3 := RGBToColor(Round(R / K3), Round(G / K3), Round(B / K3));

   If FrameHeight = double Then
      Color4 := RGBToColor(Round(R / K4), Round(G / K4), Round(B / K4))
   Else
      Color4 := Color3;

   FStart.X := 0;
   FStart.Y := 0;
   FEnd.X := Width - 1;
   FEnd.Y := Height - 1;

   BStart.X := FrameSize;
   BStart.Y := FrameSize;
   BEnd.X := Width - FrameSize - 1;
   BEnd.Y := Height - FrameSize - 1;

   If (FrameStyle = Raised) Or (FrameStyle = Relief) Then
   Begin
      DrawShadow(FStart, FEnd, Color1, Color4);

      If FrameHeight = double Then
      Begin
         FStart.X := FStart.X + 1;
         FStart.Y := FStart.Y + 1;
         FEnd.X := FEnd.X - 1;
         FEnd.Y := FEnd.Y - 1;
         DrawShadow(FStart, FEnd, Color2, Color3);
      End;
   End;

   If (FrameStyle = Lowered) Or (FrameStyle = Relief) Then
   Begin
      DrawShadow(BStart, BEnd, Color3, Color1);
      If FrameHeight = double Then
      Begin
         BStart.X := BStart.X + 1;
         BStart.Y := BStart.Y + 1;
         BEnd.X := BEnd.X - 1;
         BEnd.Y := BEnd.Y - 1;
         DrawShadow(BStart, BEnd, Color4, Color2);
      End;
   End;
End;

Procedure TplLCDLine.DrawShadow(StartP, EndP: TPoint; LineColor1, LineColor2: TColor);
Begin
   With FBitMap.Canvas Do
   Begin
      Pen.Color := LineColor1;
      MoveTo(EndP.X, StartP.Y);
      LineTo(StartP.X, StartP.Y);
      LineTo(StartP.X, EndP.Y);

      Pen.Color := LineColor2;
      MoveTo(EndP.X, StartP.Y);
      LineTo(EndP.X, EndP.Y);
      LineTo(StartP.X - 1, EndP.Y);
   End;
End;

Procedure TplLCDLine.DrawGrid;
Var
   y, x: integer;
   NRow, NCol: integer;
Begin

   NRow := FRowCount;
   NCol := (FColCount + 1) * FCharCount - 1;

   For y := 0 To NRow - 1 Do
      For x := 0 To NCol - 1 Do
         DrawCell(y, x, CellColorOff);

End;

Procedure TplLCDLine.DrawSpace;
Var
   y, x: integer;
   NRow, NCol: integer;
   i: integer;
Begin

   NRow := FRowCount;
   NCol := (FColCount + 1) * FCharCount - 1;

   For y := 0 To NRow - 1 Do
   Begin
      i := 0;
      For x := 0 To NCol - 1 Do
      Begin
         If i = 5 Then
         Begin
            DrawCell(y, x, BoardColor);
            i := 0;
         End
         Else
            i := i + 1;
      End;
   End;

End;

Procedure TplLCDLine.DrawText;
Var
   x, y, c: integer;
   CodChar: integer;
   CellRow: integer;
   CellOn: boolean;
Begin

   FLenText := Length(Text);

   For c := 1 To FLenText Do
   Begin
      CodChar := Ord(Text[c]);

      For y := 0 To 6 Do
      Begin
         CellRow := CharAr[CodChar, y];
         For x := 0 To 4 Do
         Begin
            CellOn := CellRow And (1 Shl (5 - x - 1)) > 0;
            If CellOn Then
               DrawChar(y, x + FOffset, c);
         End;
      End;
   End;

   If CharSpace Then
      DrawSpace;
End;

Procedure TplLCDLine.DrawChar(Row, Col, NChar: integer);
Begin

   Col := Col + ((FColCount + 1) * (NChar - 1));
   If Col > FGlobalColCount - 1 Then
      Exit;
   If Col < 0 Then
      Exit;
   DrawCell(Row, Col, CellColorOn);

End;

Procedure TplLCDLine.DrawCellsOn;
Var
   x, y: integer;
   Col: integer;
Begin

   For y := 0 To (FRowCount - 1) Do
      For x := 0 To (FCountOn - 1) Do
         If CellsOn[y, x] Then
         Begin
            Col := x + Offset;
            If (Col >= 0) And (Col < FGlobalColCount) Then
               DrawCell(y, Col, CellColorOn);
         End;
End;

Procedure TplLCDLine.DrawCell(Row, Col: integer; ColorCell: TColor);
Var
   CellR: TRect;
Begin
   CellR.Left := FrameSize + FBoardWidth + (CellSize + CellSpace) * Col;
   CellR.Top := FrameSize + FBoardHeight + (CellSize + CellSpace) * Row;
   CellR.Right := CellR.Left + CellSize;
   CellR.Bottom := CellR.Top + CellSize;

   With FBitMap.Canvas Do
   Begin
      Pen.Color := ColorCell;
      Brush.Color := ColorCell;

      If CellShape = stSquare Then
         FillRect(CellR)
      Else
         Ellipse(CellR.Left, CellR.Top, CellR.Right, CellR.Bottom);
   End;
End;

Procedure TplLCDLine.OutBorder;
Begin
   Canvas.Draw(0, 0, FBitMap);
End;

Procedure TplLCDLine.Paint;
Begin
   Inherited;
   DrawCalc;
   DrawBorder;
   DrawGrid;
   DrawText;
   DrawCellsOn;
   OutBorder;
End;

Procedure TplLCDLine.SetBoardColor(Const Value: TColor);
Begin
   If Value = BoardColor Then
      Exit;
   FBoardColor := Value;
   invalidate;
End;

Procedure TplLCDLine.SetCellColorOff(Const Value: TColor);
Begin
   If Value = CellColorOff Then
      Exit;
   FCellColorOff := Value;
   invalidate;
End;

Procedure TplLCDLine.SetCelColorlOn(Const Value: TColor);
Begin
   If Value = CellColorOff Then
      Exit;
   FCellColorOn := Value;
   invalidate;
End;

Procedure TplLCDLine.SetCellSize(Const Value: integer);
Begin
   If Value = CellSize Then
      Exit;
   FCellSize := Value;
   invalidate;
End;

Procedure TplLCDLine.SetCellSpace(Const Value: integer);
Begin
   If Value = CellSpace Then
      Exit;
   FCellSpace := Value;
   invalidate;
End;

Procedure TplLCDLine.SetCellShape(Const Value: TCellShape);
Begin
   If Value = CellShape Then
      Exit;
   FCellShape := Value;
   invalidate;
End;

Procedure TplLCDLine.SetFrameColor(Const Value: TColor);
Begin
   If Value = FrameColor Then
      Exit;
   FFrameColor := Value;
   invalidate;
End;

Procedure TplLCDLine.SetFrameColorStyle(Const Value: TFrameColorStyle);
Begin
   If Value = FrameColorStyle Then
      Exit;
   FFrameColorStyle := Value;
   invalidate;
End;

Procedure TplLCDLine.SetFrameHeight(Const Value: TFrameHeight);
Begin
   If Value = FrameHeight Then
      Exit;
   FFrameHeight := Value;
   invalidate;
End;

Procedure TplLCDLine.SetFrameSize(Const Value: integer);
Begin
   If Value = FrameSize Then
      Exit;
   FFrameSize := Value;
   invalidate;
End;

Procedure TplLCDLine.SetFrameStyle(Const Value: TFrameStyle);
Begin
   If Value = FrameStyle Then
      Exit;
   FFrameStyle := Value;
   invalidate;
End;

Procedure TplLCDLine.SetCharSpace(Const Value: boolean);
Begin
   If Value = CharSpace Then
      Exit;
   FCharSpace := Value;
   invalidate;
End;

Procedure TplLCDLine.SetOffset(Const Value: longint);
Begin
   If Value = Offset Then
      Exit;
   FOffset := Value;
   invalidate;
End;

Procedure TplLCDLine.SetText(Const Value: String);
Begin
   If Value = Text Then
      Exit;
   FText := Value;
   invalidate;
End;

Procedure TplLCDLine.SetLAutoSize(Const Value: boolean);
Begin
   If Value = AutoSize Then
      Exit;
   FAutoSize := Value;
   invalidate;
End;

Function TplLCDLine.GetCharCount: longint;
Begin
   DrawCalc;
   Result := FCharCount;
End;

Function TplLCDLine.GetGlobalColCount: longint;
Begin
   DrawCalc;
   Result := FGlobalColCount;
End;

Function TplLCDLine.GetCellsOn(Row, Col: integer): boolean;
Begin
   Result := FCellsOn[Row, Col];
End;

Procedure TplLCDLine.SetCellsOn(Row, Col: integer; Const Value: boolean);
Begin
   If (Row < 0) Or (Row >= FRowCount) Then
      Raise Exception.Create('Index of a Row outside an array');
   If (Col < 0) Or (Col >= FCountOn) Then
      Raise Exception.Create('Index of a Col outside an array');
   FCellsOn[Row, Col] := Value;
End;

End.
